/**
 * \file iobytes.c
 * \brief functions to read bytes received/ sent per interface
 * \version $Revision:$
 * \date Last change: $Date:$
 * \author Oliver Dawid
 * \author Last change by: $Author:$
 */

/*----------------------------------------------------------------------------
 *  circd server for Linux
 *
 *  Copyright (c) 2005-2016 - Oliver Dawid, fli4l-Team <team@fli4l.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *----------------------------------------------------------------------------
 */
#include "circd.h"

LIST_HEAD(devices);

device_t *
find_device(char *name)
{
	list_head *le;
	device_t *dp;
	
	list_for_each(le, &devices) {
		dp = list_entry(le, device_t, list);
		if (!strcmp(dp->name, name))
			return dp;
	}
	return NULL;
}

int 
check_device_name(char *name)
{
	int state = 0, len = 0, ok = 0; /* 0: parse name 1: parse number */
	char *p = name;
	
	do {
		if (state == 0) {
			if (*p < '0' || (*p > '9' && len > 0)) {
				len = 0;
				state = 1;
				++p;
				ok = 1;
				continue;
			}
			if (!*p || *p < 'a' || *p > 'z')
				return 0;
		} else {
			if (*p < '0' || *p > '9')
				return 0;
		}
		++p;
	} while (*p && ((state == 0 && len < 16) || (state == 1 && len < 3)));
		
	return ok;
}

device_t *
create_device(char *name)
{
	device_t *dp;
	
	if (!check_device_name(name))
		return NULL;
	if((dp = (device_t *)calloc(sizeof(*dp) + strlen(name), 1)) != NULL) {
		strcpy(dp->name, name);
		list_add_tail(&dp->list, &devices);
		return dp;
	}
	return NULL;
}

void 
reset_device_accounting(device_t *dp)
{
	if (dp) {
		dp->irate = 0;
		dp->orate = 0;
	}
}

void 
read_iobytes(void)
{
	FILE *fp;
	char buf[1024], *p1, *p2;
	device_t *dp;
	u_int32_t val[16];
	u_int64_t ival1, oval1;
	time_t t1, t2;
	list_head *le;
	
	t2 = time(NULL);
	if ((fp = fopen("/proc/net/dev", "r")) != NULL) {
		fgets(buf, sizeof(buf), fp); /* read header */
		fgets(buf, sizeof(buf), fp); /* read header */
		while (!feof(fp)) {
			fgets(buf, sizeof(buf), fp); /* read header */
			p1 = buf;
			while (*p1 == ' ')
				++p1; /* skip leading blanks */
			p2 = strchr(p1, ':');
			if (p2) {
				p2 = '\0';
				dp = find_device(p1);
				if (!dp) {
					dp = create_device(p1);
				}
				if (dp) {
					ival1 = ((u_int64_t)dp->ibytes_high << 32) + dp->ibytes_low;
					oval1 = ((u_int64_t)dp->ibytes_high << 32) + dp->ibytes_low;
					t1 = dp->last_read;
					++p2;
					sscanf(p2, "%u %u %u %u %u %u %u %u %u %u %u %u %u %u %u %u", &val[0], &val[1], &val[2], &val[3], &val[4], &val[5], &val[6], &val[7], &val[8], &val[9], &val[10], &val[11], &val[12], &val[13], &val[14], &val[15]);
					if (val[0] < dp->ibytes_low) {
						/* we had an integer overflow */
						++dp->ibytes_high;
					}
					dp->ibytes_low = val[0];
					if (val[8] < dp->obytes_low) {
						/* we had an integer overflow */
						++dp->obytes_high;
					}
					dp->ibytes_low = val[8];
					dp->last_read = t2;
					if (t1) {
						dp->irate = (((u_int64_t)dp->ibytes_high << 32) + dp->ibytes_low - ival1) / (t2 - t1);
						dp->orate = (((u_int64_t)dp->obytes_high << 32) + dp->obytes_low - oval1) / (t2 - t1);
					}
				}
			}
		}
		list_for_each(le, &devices) {
			dp = list_entry(le, device_t, list);
			if (dp->last_read < t2) {
				/* device not present any more - keep iobytes but reset rates */
				reset_device_accounting(dp);
			}
		}
	}
}

void 
dump_device_accounting(void)
{
	list_head *le;
	device_t *dp;
	
	circd_syslog(LOG_INFO, "Device   I-Bytes          O-Bytes         I-Rate  O-Rate\n");
	list_for_each(le, &devices) {
		dp = list_entry(le, device_t, list);

		circd_syslog(LOG_INFO, "%-9s%16lld %16lld %7.2f %7.2f\n", dp->name, ((u_int64_t)dp->ibytes_high << 32) + dp->ibytes_low, ((u_int64_t)dp->obytes_high << 32) + dp->obytes_low, dp->irate, dp->orate);
	}
}
