/**
 * \file isdn.c
 * \brief isdn specific control functions
 * \version $Revision:$
 * \date Last change: $Date:$
 * \author Oliver Dawid
 * \author Last change by: $Author:$
 */

/*----------------------------------------------------------------------------
 *  circd server for Linux
 *
 *  Copyright (c) 2005-2016 - Oliver Dawid, fli4l-Team <team@fli4l.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *----------------------------------------------------------------------------
 */

#include "circd.h"

#if 0
static void isdn_calc_rates(circuit_t *circ);
static void read_isdn_iobytes(void *context);
static void isdn_calc_rates(circuit_t *circ);
static void isdn_calc_iobytes(circuit_t *circ);
#endif

type_functable_t ippp_tab = {
	NULL, /* init */
	NULL, /* start */
	NULL, /* dial */
	NULL, /* hangup */
	NULL, /* stop */
	NULL, /* pause */
	NULL, /* cont */
	NULL, /* dialmode */
	NULL, /* hup_timeout */
	NULL, /* status */
	NULL, /* calc_rates */
	NULL, /* calc_iobytes */
	NULL, /* read_persistence */
	NULL  /* write_persistence */
};

type_functable_t isdn_tab = {
	NULL, /* init */
	NULL, /* start */
	NULL, /* dial */
	NULL, /* hangup */
	NULL, /* stop */
	NULL, /* pause */
	NULL, /* cont */
	NULL, /* dialmode */
	NULL, /* hup_timeout */
	NULL, /* status */
	NULL, /* calc_rates */
	NULL, /* calc_iobytes */
	NULL, /* read_persistence */
	NULL  /* write_persistence */
};

#if 0
static struct {
	/** This array structure holds the statistics returned by the IIOCGETCPS ioctl
 	 */
	struct {
  		unsigned long ibytes;
  		unsigned long obytes;
	} io_bytes[ISDN_MAX_CHANNELS * 2];
	struct timeval tv1;
	struct timeval tv2;
} io_info;

static int isdninfo_fd = -1;

static void isdn_init(void)
{
	if ((isdninfo_fd = open ("/dev/isdninfo", O_RDONLY)) == -1) {
		circd_syslog(LOG_ERR, "Cannot open /dev/isdninfo: %s", strerror(errno));
		return;
	}
}

static void isdn_calc_rates(circuit_t *circ)
{
	time_table_t *tt;
	u_int64_t bytes, t1, t2;
	
	if (circ->current_tab == -1)
		return;

	if (isdninfo_fd == -1)
		return;

	t1 = io_info.tv1.tv_sec * 10 + io_info.tv1.tv_usec / 100000;
	t2 = io_info.tv2.tv_sec * 10 + io_info.tv2.tv_usec / 100000;
	tt = &circ->time_tab[circ->current_tab];

	/* calculate rates for imon clients */
	bytes =  io_info.io_bytes[circ->dev_idx1 + ISDN_MAX_CHANNELS].ibytes;
	if (io_info.io_bytes[circ->dev_idx1].ibytes > bytes)
		bytes += 0x100000000ull;
	circ->irate = (bytes - io_info.io_bytes[circ->dev_idx1].ibytes) / (t2 - t1);

	bytes =  io_info.io_bytes[circ->dev_idx1 + ISDN_MAX_CHANNELS].obytes;
	if (io_info.io_bytes[circ->dev_idx1].ibytes > bytes)
		bytes += 0x100000000ull;
	circ->orate = (bytes - io_info.io_bytes[circ->dev_idx1].obytes) / (t2 - t1);

	if (circ->dev_idx2 > -1) { /* add bytes from slave device if existing */
		/* calculate rates for imon clients */
		bytes =  io_info.io_bytes[circ->dev_idx2 + ISDN_MAX_CHANNELS].ibytes;
		if (io_info.io_bytes[circ->dev_idx2].ibytes > bytes)
			bytes += 0x100000000ull;
		circ->irate += (bytes - io_info.io_bytes[circ->dev_idx2].ibytes) / (t2 - t1);
	
		bytes =  io_info.io_bytes[circ->dev_idx1 + ISDN_MAX_CHANNELS].obytes;
		if (io_info.io_bytes[circ->dev_idx2].ibytes > bytes)
			bytes += 0x100000000ull;
		circ->orate += (bytes - io_info.io_bytes[circ->dev_idx2].obytes) / (t2 - t1);
	}
}

static void isdn_calc_iobytes(circuit_t *circ)
{
	time_table_t *tt;
	u_int64_t bytes;
	
	if (circ->current_tab == -1)
		return;

	if (isdninfo_fd == -1)
		return;

	tt = &circ->time_tab[circ->current_tab];
	/* fill in session ibytes and obytes for time_table record */
	bytes = io_info.io_bytes[circ->dev_idx1 + ISDN_MAX_CHANNELS].ibytes;
	while (tt->sess_ibytes > bytes)
		bytes += 0x100000000ull;
	tt->sess_ibytes = bytes;
	
	bytes = io_info.io_bytes[circ->dev_idx1 + ISDN_MAX_CHANNELS].obytes;
	while (tt->sess_obytes > bytes)
		bytes += 0x100000000ull;
	tt->sess_obytes = bytes;

	if (circ->dev_idx2 > -1) { /* add bytes from slave device if existing */
		/* fill in session ibytes and obytes for time_table record */
		bytes = io_info.io_bytes[circ->dev_idx2 + ISDN_MAX_CHANNELS].ibytes;
		while (tt->sess_ibytes > bytes)
			bytes += 0x100000000ull;
		tt->sess_ibytes += bytes;
		
		bytes = io_info.io_bytes[circ->dev_idx2 + ISDN_MAX_CHANNELS].obytes;
		while (tt->sess_obytes > bytes)
			bytes += 0x100000000ull;
		tt->sess_obytes += bytes;
	}
}

static void read_isdn_iobytes(void *context)
{
	memcpy(io_info.io_bytes, &io_info.io_bytes[ISDN_MAX_CHANNELS], sizeof(io_info.io_bytes) / 2);
	io_info.tv1 = io_info.tv2;
	gettimeofday(&io_info.tv2, NULL);
	if (ioctl (isdninfo_fd, IIOCGETCPS, &io_info.io_bytes[ISDN_MAX_CHANNELS]) == -1)
		circd_syslog(LOG_ERR, "Could not read io_bytes from isdninfo_fd: %s", strerror(errno));

	set_timer(TIMEOUT_CALC_IOBYTES, read_isdn_iobytes, NULL);
}
#endif
