/*!
 * $Id: button.c 34658 2014-11-06 23:10:34Z kristov $
 *
 * @file    button.c
 * @brief   implementation of button handler for hwsuppd deamon
 *
 * @author  Carsten Spie&szlig; fli4l@carsten-spiess.de
 * @date    17.08.2013
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <syslog.h>

#include "hwsuppd.h"

static const char* s_strGpioButton = "/var/run/hwsuppd/hwsupp-button%d/value";

#define verify_max_buttons(num) do { if (num < 0 || num >= g_data.nButton) return; } while(0)

/*!
 * @fn          static int get_button (int num)
 * @brief       get button status
 * @details     read from file matching strMultiButton
 * @param[in]   num
 *              button number
 * @return      button status
 * @retval      1
 *              button released
 * @retval      0
 *              button pressed
 */
static int get_button (int num)
{
    int fd;
    unsigned char state;
    char devname[80];
    snprintf(devname, sizeof(devname), s_strGpioButton, num + 1);

    if ((fd = open(devname, O_RDONLY)) < 0)
    {
        log_printf(LOG_ERR, "error opening %s: %s, ignoring button", devname, strerror(errno));
        return g_data.statusButton[num];
    }

    if (read(fd, &state, sizeof(state)) < 0)
    {
        close(fd);
        log_printf(LOG_ERR, "error reading button %d: %s, ignoring button", num, strerror(errno));
        return g_data.statusButton[num];
    }

    close(fd);
    return (state == '0')?0:1;
}

/*!
 * @fn          void button_read (int num)
 * @brief       read button
 * @details     read button state and execute command when button is pressed
 * @param[in]   num
 *              button number
 */
void button_read (int num)
{
    // read status of pushbutton switch
    // 1 = released
    // 0 = pressed

    int old_status;

    verify_max_buttons(num);

    if (0 == g_data.nButton)
    {
        return;
    }

    old_status = g_data.statusButton[num];

	g_data.statusButton[num] = get_button(num);
	if ((1 == old_status) && (0 == g_data.statusButton[num]))
	{
		execute_command (num);
	}
}

/*_oOo*/
