/*
 *  leds-apu.c - LED support for PC-Engines APU board
 *
 *  Copyright (c) 2014  Carsten Spiess <fli4l at carsten-spiess.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/leds.h>
#include <linux/platform_device.h>

static struct gpio_led apu_leds[] = {
	{
		.name = "apu::1",
		.gpio = 253,
		.default_trigger = "none",
		.active_low = 1,
	},
	{
		.name = "apu::2",
		.gpio = 254,
		.default_trigger = "none",
		.active_low = 1,
	},
	{
		.name = "apu::3",
		.gpio = 255,
		.default_trigger = "none",
		.active_low = 1,
	},
};

static struct gpio_led_platform_data apu_leds_data = {
	.num_leds = ARRAY_SIZE (apu_leds),
	.leds = apu_leds,
};

static void leds_apu_release (struct device *dev)
{
}

static struct platform_device apu_leds_dev = {
	.name = "leds-gpio",
	.id = -1,
	.dev = {
		.release = leds_apu_release,
		.platform_data = &apu_leds_data,
	}
};

static int __init leds_apu_init (void)
{
	int ret;
	ret = platform_device_register (&apu_leds_dev);
	if (ret) {
		pr_err ("adding leds device failed\n");
	}
	return ret;
}

static void __exit leds_apu_exit (void)
{
	platform_device_unregister (&apu_leds_dev);
}

MODULE_AUTHOR ("Carsten Spiess <fli4l at carsten-spiess.de>");
MODULE_DESCRIPTION ("GPIO-LED driver for PC-Engines APU");
MODULE_LICENSE ("GPL");


module_init (leds_apu_init);
module_exit (leds_apu_exit);


