/*
 *  leds-net5501.c - LED support for Soekris net5501 board
 *
 *  Copyright (c) 2014  Carsten Spiess <fli4l at carsten-spiess.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/leds.h>
#include <linux/platform_device.h>

static struct gpio_led net5501_leds[] = {
	{
		.name = "net5501::error",
		.gpio = 6,
		.default_trigger = "none",
		.active_low = 1,
	},
};

static struct gpio_led_platform_data net5501_leds_data = {
	.num_leds = ARRAY_SIZE (net5501_leds),
	.leds = net5501_leds,
};

static void leds_net5501_release (struct device *dev)
{
}

static struct platform_device net5501_leds_dev = {
	.name = "leds-gpio",
	.id = -1,
	.dev = {
		.release = leds_net5501_release,
		.platform_data = &net5501_leds_data,
	}
};

static int __init leds_net5501_init (void)
{
	int ret;
	ret = platform_device_register (&net5501_leds_dev);
	if (ret) {
		pr_err ("adding leds device failed\n");
	}
	return ret;
}

static void __exit leds_net5501_exit (void)
{
	platform_device_unregister (&net5501_leds_dev);
}

MODULE_AUTHOR ("Carsten Spiess <fli4l at carsten-spiess.de>");
MODULE_DESCRIPTION ("GPIO-LED driver for Soekris net5501");
MODULE_LICENSE ("GPL");


module_init (leds_net5501_init);
module_exit (leds_net5501_exit);


